//*************************************************************************************************************
//*************************************************************************************************************
// File: offcmds.cpp
//*************************************************************************************************************
//*************************************************************************************************************

#include "stdafx.h"
#include "sowaiinf.h"

#include "../sowmod/xunit.h"
#include "../sowmod/xunitdef.h"
#include "../sowmod/xtables.h"

#include "SOWWLMOD.h"

#include <list>

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

#define BUFF_CHECK(val) CXUtil::MultSpec(val,11,10)
#define DISTCAV1AUTOMELEE 100



//*************************************************************************************************************
//*************************************************************************************************************
// Function: IsColumn
//*************************************************************************************************************
//*************************************************************************************************************
static bool IsColumn( CXUnit base )
{
//check if the formation is one of column types
	if (base.IsFormType(eFTAssault) || base.IsFormType(eFTColumnHalf) || base.IsFormType(eFTColumnFull) 
		|| base.IsFormType(eFTMarch) )
		return true;


	return false;
}
//new infantry
//*************************************************************************************************************
//*************************************************************************************************************
// Function: IsLine
//*************************************************************************************************************
//*************************************************************************************************************
static bool IsLine( CXUnit base )
{
//check if the formation is one of line types
	if (base.IsFormType(eFTFight) || base.IsFormType(eFTLine))
		return true;

	return false;
}
// new square
//*************************************************************************************************************
//*************************************************************************************************************
// Function: IsSquareType
//*************************************************************************************************************
//*************************************************************************************************************
static bool IsSquareType( CXUnit base, bool GroupSquare, bool OnlyGroup )
{
//function check if the formation is a square type to exclude from particolar checks; the group square parameter
// is for check particular formations like regimental squares, ordre mixte, etc
// only group is for only group formations
	bool bRes = false;

	if (base.UnitType()!=eUnitInf)  //check if infantry
		return bRes;

	if (OnlyGroup==false)  //yes for battalion information
	{
		if (base.IsFormType(eFTSquare))  //battalion square
		{
			bRes = true;
		}

	}

	if (GroupSquare==true)  //I check also group formation similar to square (no change
	{
		if (base.Leader().IsFormType(eFTSquare))  //regimental square
		{
			bRes = true;
		}

	}


	return bRes;

}
//*************************************************************************************************************
//*************************************************************************************************************
// Function: CheckGunRedeploy
//*************************************************************************************************************
//*************************************************************************************************************
static bool CanGunRedeploy( CXOff off, CXVec Dest )
{
	int i;
	CXUnit subunit;

//--not deploy in one location where there is already a gun of the same battery
// avoid overlapping of batteries
	for (i=0;i<off.NumSubs();i++)
	{
		subunit = off.Sub(i);
		if (subunit.Loc().DistYds(Dest)<=5)
			return false;
	}
	
	return true;

}
//*************************************************************************************************************
//*************************************************************************************************************
// Function: IsSkirmisher
//*************************************************************************************************************
//*************************************************************************************************************
static bool IsSkirmisher( CXUnit base )
{
//check if the formation is one of skirmishers types

	if (base.IsFormType(eFTSkirmish) || base.IsFormType(eFTAltSkirmish))
		return true;


	return false;
}
//*************************************************************************************************************
//*************************************************************************************************************
// Function: SquareFriendOverlap
//*************************************************************************************************************
//*************************************************************************************************************
static bool SquareFriendOverlap( CXUnit base )
{

	CXUnit sold;
	int dist;
	int diff;
	SXComm comm;
	CXUnit closest;
	int flagdist;
//check if I have space to form a square

//I check if I have friend to near to me
	base.FriendBeg();

	while ( base.FriendNext( sold, dist ) )
	{
		if (sold.IsOfficer())  //no officers
			continue;
		
		if (sold.UnitType()!=eUnitInf && sold.UnitType()!=eUnitCav)  //no units different from cavalry and infantry
			continue;
		
		if (sold.UnitType()==eUnitInf && IsSkirmisher(sold) )  //no skirmishers
			continue;

		if (base.DoesShareHex(sold) && IsSquareType(sold,true,false))  //if overlap with friend in square no square
			return true;

		flagdist = base.Loc().DistYds(sold.Loc());

		if (base.DoesShareHex(sold) && flagdist<=40)  //if overlap with friend and the center is under the 40 yards no square
			return true;

		diff = base.Pos().GetAngleDiff(sold.Loc(),true);   //if friend behind at 40 yards no square
		if (diff>=90 && diff<=270 && flagdist<=40)
			return true;

	
	}

	return false;

}
//new infantry
//*************************************************************************************************************
//*************************************************************************************************************
// Function: FearCheck
//*************************************************************************************************************
//*************************************************************************************************************
static int FearCheck( CXUnit base, CXUnit sold )
{
	int defender;
	int attacker;
	int diff = base.Pos().GetAngleDiff(sold.Loc(),true); 
	int sidemultiplier = 1;
	int attackertypemultiplier = 1;
	int defendertypemultiplier = 1;
	int attackerrunmultiplier = 1;
	int defenderrunmultiplier = 1;
	int result = 0;

//fear check -> calculation for pre-melee reaction
//--- base = defender
//--- sold  = attacker

	if (!IsSquareType(base,true,false))  //attacks on flanks multipliers (squares have not flanks)
	{
		if (diff>80 && diff<280)
			sidemultiplier = 2;
		if (diff>135 && diff<225)
			sidemultiplier = 3;
	}

	if (base.UnitType()==eUnitCav)  //cavalry multiplier
	{
		defendertypemultiplier *= 10;
		if (base.State()==eUnitRun || base.State()==eUnitCharge)  //run charge is a additional factor
			 defenderrunmultiplier *= 4;
	}

	if (sold.UnitType()==eUnitCav) //cavalry multiplier
	{
		attackertypemultiplier *= 10;
		if (sold.State()==eUnitRun || sold.State()==eUnitCharge) //run charge is a additional factor
			 attackerrunmultiplier *= 4;
	}

	defender = ((base.NumMen()*defendertypemultiplier)/100);
	if (IsSkirmisher(base))  //skirmishes are valid only half
		defender /= 2;

	attacker = ((sold.NumMen()*attackertypemultiplier)/100);
	if (IsSkirmisher(sold)) //skirmishes are valid only half
		attacker /= 2;

	defender =  defender + base.Morale() + (base.MorUnitBon()/100) + base.VolleyStatus() 
				- (base.UnderFire() + base.UnderFire1() + base.UnderRecentFire()) //fire penalty
				+ (base.IsHighGrnd()*4) + (base.OnDefTerr()*2) - base.IsFlanked() + defenderrunmultiplier;

	attacker = attacker + sold.Morale() + (sold.MorUnitBon()/100) 
			   - (sold.UnderFire() + sold.UnderFire1() + sold.UnderRecentFire())  //fire penalty
			   + base.VolleyStatus() + sold.CanRun() + (sold.IsHighGrnd()*2) - sold.IsFlanked() + attackerrunmultiplier;

	attacker *= sidemultiplier;

	if (defender==0)  //for avoid divide by zero exception
		defender = 1;

	result = attacker/defender;

	if ((attacker%defender)*2>=defender)  //commercial rounding
		result += 1;

	return result;

}
//*************************************************************************************************************
//*************************************************************************************************************
// Function: FriendOverlap
//*************************************************************************************************************
//*************************************************************************************************************
static bool FriendOverlap( CXUnit base )
{
	CXUnit sold;
	int dist;
	int diff;
	SXComm comm;
	CXUnit closest;
	int mindist = 0;

//I check if I have friend to near to me to decide how to avoid the overlap
	base.FriendBeg();

	while ( base.FriendNext( sold, dist ) )
	{
		if (sold.IsOfficer())  //no officer
			continue;
		
		if (sold.UnitType()!=eUnitInf && sold.UnitType()!=eUnitCav)  //no units different from infantry or cavalry
			continue;
		
		if (sold.UnitType()==eUnitInf && IsSkirmisher(sold) )  //no skirmishers
			continue;

		if (sold.State()!=eUnitStand && sold.State()!=eUnitShoot && sold.State()!=eUnitAdvance)  //no friend in movement
			continue;

		if (base.DoesShareHex(sold))  //if I share hex with a friend ahead of me
		{
			diff = base.Pos().GetAngleDiff(sold.Loc(),true);
			if ((diff<= 90 || diff >= 270))
			{
				if (closest==NULL || dist<mindist)
				{
					closest = sold;
					mindist = dist;
				}
			}
		}
	}

	if (closest!=NULL)   //I move behind of 40 yards in the case of square or long range distance in the case of lines\columns
	{
		if (IsSquareType(closest,true,false))
		{
			comm.Vec (true, base.GetLocBehind(40));
			comm.Dir (true, base.Dir());
			comm.Comm(eComMoveDir);
			base.Command(&comm);
			return true;
		}
		comm.Vec (true, closest.GetLocBehind(base.LongRangeYds()));
		comm.Dir (true, closest.Dir());
		comm.Comm(eComMoveDir);
		base.Command(&comm);
		return true;
	}


	return false;

}
////*************************************************************************************************************
////*************************************************************************************************************
//// Function: FriendAheadMe
////*************************************************************************************************************
////*************************************************************************************************************
static bool FriendAheadMe( CXUnit base )
{
	CXUnit sold;
	int dist;
	int diff;
	int flagdist;


//I check if I have friend to near to me for wheel
	base.FriendBeg();

	while ( base.FriendNext( sold, dist ) )
	{
		
		if (sold.IsOfficer())  //no officer
			continue;
	
		if (dist>base.MaxRangeYds())  //out of max weapon range
			continue;
		
		if (sold.UnitType()!=eUnitInf && sold.UnitType()!=eUnitCav)  //only infantry or cavalry
			continue;

		if (sold.State()!=eUnitStand && sold.State()!=eUnitShoot && !sold.Retreating() && sold.State()!=eUnitFallback    //don't check moving friends
			&& sold.State()!=eUnitAdvance)
			continue;

		if (IsSkirmisher(sold))  //no skirmishers
			continue;

		diff = base.Pos().GetAngleDiff(sold.Loc(),true);
		if ((diff<= 45 || diff >= 315))
		{
			return true;  //if in the frontal arc is always blocking
		}
		if ((diff<= 60 || diff >= 300))
		{			
			if (base.Dir()==sold.Dir())  //same diretion path blocked
				return true;
			flagdist = base.Loc().DistYds(sold.Loc());       //check of distance betweent the flags and the minimal range
			if (dist>base.LongRangeYds() && flagdist>dist)
				continue;
			if (!base.ValidWalk(base.GetLocAhead(dist)) || !base.ValidWalk(base.GetLocAhead(flagdist)))  //check validity at both distances
				return true;
		}

	}


	return false;

}


//*************************************************************************************************************
//*************************************************************************************************************
// Function: SplitCompany
//*************************************************************************************************************
//*************************************************************************************************************
bool SplitCompany( CXUnit base )
{
	int i;
	CXOff off = base.Leader();
	//check if the battalion has already a detached company

	if (off==NULL || off.NumSubs()==0)
		return true;

	if (base.NumMen()<=450)  //only big battalions can have skirmishers
	{
		return true;
	}

	if (!base.CanRun())  //only if the parent can run I don't need tired skimishers
	{
		return true;
	}

	for ( i = 0; i < off.NumSubs(); ++i )
	{
		CXUnit sub = off.Sub(i);

		if (base.GetID()==sub.Split())    //I check if there're splitted unit and what of them has base as parent
			return true;

	}

	return false;
}
//-------------new skirmishers----------------------

//*************************************************************************************************************
//*************************************************************************************************************
// Function: GetCommand
//*************************************************************************************************************
//*************************************************************************************************************

bool GetInfCommand( CXOff lead, int state, SXComm &comm, CXUnit unit )
{
	bool btarg;

	CXVec pt;
	CXVec floc;

	CXUnit targ;

//-----new fort------------
	CXVec dloc;
	bool SkirmishersOn = false;
	bool enemytoonear = false;
	int ratio = 0;
	int diff;
	bool pathblocked = false;
//---new fort--------------


	int frange = 0;
	int dist   = 1000000;

	const TStance *stance;

	std::list<CXUnit> targs;

	comm.Comm( eComMax );

	if ( !lead.HasStance() || lead.PlyrCtrl() )
		return false;

	stance = lead.GetStance();

	if ( !unit.InFort() )      //check free fort near
	{
		frange = stance->fortrange;
		floc   = CXUtil::FindClosestFort( unit, frange );
	}

//------ratio check------------------------
//----checking of force to make a melee attack
	if (unit.Closest())
		ratio = FearCheck(unit.Closest(),unit);
//----------------------------------------------

	if ( state == eSTStandNoTarget )                //situation stand no target for infantry
	{
		if ( unit.Morale() <= MORAL_CANTWHEEL || unit.PlyrCtrl() )
			return false;

		btarg = lead.HasTargets();

		if ( !btarg && lead.OrdersState() != eOrdersStay )
			targ = lead.GetSaveClosest( dist );

		// let the brigade move as one in this case
		if ( !lead.CurrTarg() && !lead.Moving( true ) && targ.Valid() )
			return false;


		//new infantry : i want avoid manouvers differents from the normal advance with the enemy too near
		if (unit.EnemyClose() && unit.EnemyCloseYds()<=(BUFF_CHECK(unit.MaxRangeYds())))   
		{
			enemytoonear = true;
		}		


		if ( !btarg || unit.EnemyCloseYds()<(unit.MaxRangeYds() * 4) || (lead.EnemyClose() && IsSkirmisher(lead.EnemyClose())))  //skirmishers activation
			 SkirmishersOn = true;
		if (  SkirmishersOn && enemytoonear )  //minimal distance check
			 SkirmishersOn = false;
//-------new skirmishers----------------------------------------------------------------------------------------------------
		if (SkirmishersOn && lead.EnemyClose() && lead.EnemyCloseYds()<(lead.MaxRangeYds() * 4) && !lead.IsFormType(eFTAltSkirmish))  //if the brigade is already in big skirmishers formation i don't detach single companies
		{
			 if (unit.Split()==-1) 
			 {
				//conditions for deploy a the skirmishers company (formation,morale,distance from the enemy, dimension,attitude, no already split unit)
				if (!unit.InFort() && !IsSkirmisher(unit) && !IsSquareType(unit,true,false) && stance->SkirmAttitude == 1 && unit.Morale() >= stance->SkirmSplit && !SplitCompany(unit) && stance->SkirmDimension!=0 )
				{
					comm.Comm(eComSplit);
					comm.Val((unit.NumMen()/stance->SkirmDimension));
					unit.Command(&comm);
//					return true;
				}
			 }
		}
///new volley-------------------
		if ( unit.VolleyStatus() == 0 && !unit.TargClose() && unit.Morale() >= stance->volley && !IsSkirmisher(unit) && !unit.InFort())     // prepare a volley if I have the correct conditions
		{
			comm.Comm(eComVolleyOn);
			return true;
		}
//------new volley
//------------new skirmishers-----------------------------------------------------

		// leader's max range is based on his subordinates
		if ( btarg || ( targ.Valid() && dist < lead.MaxRangeYds() + (lead.Personality() * 50) ) )
		{

			//check of friend ahead me
			pathblocked = FriendAheadMe(unit);

			// Flanking
			if ( unit.Morale() >= stance->flanking && lead.GetFlankDest( unit, pt, targ ) &&    //flanking movement; the location is on the flank of target
				!pathblocked && !IsSquareType(unit,true,false) )
			{
				comm.Vec (true,pt);
				comm.Comm(eComMove);
			}
			// Maintain line			
			else if ( unit.Morale() >= stance->maintainline && lead.GetLineDest( unit, pt )    //maintain line; the location is on the line according to the leader front
				&& (!unit.EnemyClose() || unit.EnemyCloseYds()>=unit.MaxRangeYds() || IsColumn(unit) || unit.EnemyClose().InFort() )
				&& !pathblocked && !IsSquareType(unit,true,true))
			{
				comm.Vec (true, pt);
				comm.Comm(eComMove);
			}
			else if ( CXUtil::GetDifficulty() != eDiffGreen )   //difficulty check, if difficulty is higher than green
			{
				if ( !targ )   //if target for the leader is not present I set the nearest enemy of unit in a determined range
				{
					if ( unit.EnemyClose().Valid() && (unit.EnemyCloseYds() < unit.MaxRangeYds() + (lead.Personality() * 50)) )
						targ = unit.EnemyClose();
				}

			 	if ( unit.Morale() >= stance->moveforward && targ.Valid() && lead.GetForwardDest( unit, pt, targ )   //move forward, respect current position
					&& !pathblocked && !IsSquareType(unit,true,false))
				{
					comm.Vec (true,pt);
					comm.Comm(eComMove);
				}
				else if ( unit.EnemyClose().Valid() && !IsSquareType(unit,true,false))   //wheeling (no square)
				{
					
					// this will cause a wheel to target here; friend not too close or in column or enemy closest in the fort
					if (unit.FriendCloseYds()>unit.LongRangeYds() || IsColumn(unit) || unit.EnemyClose().InFort())
					{
						comm.Unit( unit.EnemyClose().GetID() );
						comm.Comm(eComWheelDir);
					}
					else if (unit.FriendCloseYds()!=0 && FriendOverlap(unit))  //if overlap with friend nothing
					    return true;
					else if (unit.IsFlanked())  //if flanked I move back
					{
						comm.Vec(true,unit.GetLocBehind(unit.LongRangeYds()));
						comm.Comm(eComMove);
					}
						
				}
			}
			else if ( unit.EnemyClose().Valid() 
				&& !IsSquareType(unit,true,false))  // same as above
			{
				// this will cause a wheel to target here
					if (unit.FriendCloseYds()>unit.LongRangeYds() || IsColumn(unit) || unit.EnemyClose().InFort())
					{
						comm.Unit(unit.EnemyClose().GetID());
						comm.Comm(eComWheelDir);
					}
					else if (unit.FriendCloseYds()!=0 && FriendOverlap(unit))
					    return true;
					else if (unit.IsFlanked())
					{
						comm.Vec(true,unit.GetLocBehind(unit.LongRangeYds()));
						comm.Comm(eComMove);
					}
			}
		}
	}
	else if ( state == eSTStandShooting )   //situation when unit is shooting
	{
		if (!unit.TargFind(unit.Closest(),dist))  //target distance
			dist = INT_MAX;

		if ( lead.ShouldGetAmmo( unit ) )   //need ammo or i move in fort or i get away
		{
			if ( !floc.empty() )
			{
				comm.Comm(eComMove);
				comm.Vec (true,floc);
			}
			else
				comm.Comm(eComGetAway);
		}
		else if ( unit.Morale() <= stance->retreat && unit.MorChange() < 0 && !IsSquareType(unit,true,false))   //loss of morale retreat in fort or back
		{
			if ( !floc.empty() )
			{
				comm.Comm(eComMove);
				comm.Vec (true,floc);
			}
			else
				comm.Comm(eComRetreat);
		}
		else if ( unit.OnDefTerr() && lead.StopOnDefTerr(unit) && lead.NewDefTerr() && unit.VolleyStatus() != 1)  //i check volley status because I risk in case of false to not shot
		{
			if ( !floc.empty() )
			{
				comm.Comm(eComMove);
				comm.Vec(true,floc);
			}
			else
				return false;
		}
		else if ( unit.Morale() <= stance->fback && unit.MorChange() < 0 && !IsSquareType(unit,true,false))  //check of fallback retreat in fort or fallback
		{
			if ( !floc.empty() )
			{
				comm.Comm(eComMove);
				comm.Vec (true,floc);
			}
			else
				comm.Comm(eComFallback);
		}
//-----new infantry------------------------------------
		else if ( unit.State()==eUnitShoot && unit.VolleyStatus() == 1 )//release of volley
		{
			comm.Comm(eComVolleyOff);
		}
		else if ( !IsSquareType(unit,true,false) && lead.CanICharge(unit, stance->chargemelee) && !unit.Closest().InFort()
			&& dist < stance->InfantryCharge && unit.Closest().UnitType()!=eUnitCav && ratio>=3)  //check infantry charge from shooting condition
		{
			comm.Comm(eComCharge);
		}
		else if ( !unit.IsHighGrnd() && !IsSquareType(unit,true,false) && !unit.Closest().InFort() && unit.Morale() >= stance->moveforward && (ratio>=1
			|| unit.Closest().State()==eUnitFallback ) && unit.TargCloseYds()>=unit.LongRangeYds() && !unit.Blocked())  //advancing fire
		{
			diff = unit.Pos().GetAngleDiff(unit.Closest().Loc(),true);  //check angle for avoid lateral movement
			if ( diff <= 20 || diff >= 340) 
			{
				comm.Comm(eComAdvance);
			}
		}
//-----new infantry---------------------------------------
		else if ( !unit.PlyrCtrl() && !lead.InFort() && !IsSquareType(unit,true,false))   //not make them if the unit is under player control or in fort or in square
		{
			if ( !floc.empty() )    //move to the fort
			{
				comm.Comm(eComMove);
				comm.Vec (true,floc);
			}
			else					//asks orders from the leader
			{
				targ = unit.Closest();
				
				unit.TargFind( targ, dist );

				lead.GetOrders( dist, targ, unit );
			}
		}
	}
	else if ( state == eSTStandTargetNoAmmo )   //no ammo situation with target
	{
		if ( unit.Closest() && lead.CanICharge( unit, stance->chargenoammo ) && unit.Closest().UnitType()!=eUnitCav )  //charge
			comm.Comm(eComCharge);
		else if ( !floc.empty() )    //move in the fort
		{
			comm.Comm(eComMove);
			comm.Vec (true,floc);
		}
		else if ( !lead.InFort() )   //move away if the leader is not in the fort
			comm.Comm(eComGetAway);
	}
	else if ( state == eSTStandTargetOnlyMelee )    //no shooting only melee situation
	{
		if ( unit.Closest() && lead.CanICharge(unit) )   //charge
			comm.Comm(eComCharge);
		else if ( !floc.empty() )    //move to the fort
		{
			comm.Comm(eComMove);
			comm.Vec(true,floc);
		}
	}
// ---  new square-----------
	else if ( state == eSTStandCavalryDanger )   //cavalry danger situation
	{
		//if the unit is not already in a square formation and not in skirmishers formation
		if (!IsSquareType(unit,true,false) && !IsSkirmisher(unit))  
		{
			
			//the split check is for avoid in any case skimrisher unit can			
			if (unit.Split()==-1 && unit.NumMen()>stance->SquareMen && unit.Morale()>stance->CavDanger && ( unit.EnemyCloseYds() > stance->SquareToEnemy || unit.EnemyCloseYds()==0 ) )  //is possible form square (men number and morale and good location)
			{
				if (unit.ValidDest(unit.Loc()) && unit.State()!=eUnitMelee && !SquareFriendOverlap(unit) )  //good space I form the square
				{
					comm.Comm(eComFormType);
					comm.Val(eFTSquare);
				}
				else							//I cannot form square here so I get Away back
				{			
					comm.Comm(eComMove);
					CXVec vec = unit.GetLocBehind(150);
					if ( !unit.ValidDest(vec))
					{
						unit.GetValidDest( unit.DeP(), &vec, true );
					}
					comm.Vec(true,vec);
					unit.Command(&comm);
					comm.Comm(eComRun);
				}
			}
			else if ( !floc.empty() )   //if not I search fortress where move   
			{
				comm.Comm(eComFormType);   // move away in march formation
				comm.Val(eFTMarch);
				unit.Command(&comm);
				comm.Comm(eComMove);
				comm.Vec(true,floc);
			}
		}
		else if (lead.IsFormType(eFTSquare)) //in a brig class square do nothing
		{
			return false;
		}
		else if (unit.IsFormType(eFTSquare)) //in a reg class square (battalion square)
		{
			if ( unit.Morale() <= stance->retreat && unit.MorChange() < 0 )  //retreat check for morale loss
			{
				if ( !floc.empty() )   //move to the fort
				{
					comm.Comm(eComMove);
					comm.Vec (true,floc);
				}
				else
					comm.Comm(eComRetreat);   //retreat
			}			
			else if (unit.NumMen()<stance->SquareMen)  //losses created too many gaps not possible anymore maintain the square intact I form line again
			{
				comm.Comm(eComFormType);
				comm.Val(eFTFight);
			}
		}
		else if (IsSkirmisher(unit) && unit.Split()!=-1)  //split company of skirmishers 
		{
			if (!unit.GetCharger()) // if not charged unsplit -> if charged she will get away for the skirmisher getaway logic in THINK routine
				comm.Comm(eComUnSplit);
		}

	}
	else if ( state == eSTStandCavalryDangerEnd )  //the cavalry danger ended but I'm still in square
	{
		if (unit.EnemyClose() &&  unit.EnemyCloseYds() <= stance->SquareToEnemy )   //enemy too near to do change formation
		{
			return false;
		}
		else if ( !unit.TargClose() && unit.WantCover() && !unit.IsHighGrnd() && !unit.OnDefTerr() )  //if wantcover I move to a better location
		{
			if (unit.FindClosestFeature(SKD_DBON,stance->fortrange,dloc))  //I move in a best ground
			{
				comm.Comm(eComFormType);
				comm.Val(eFTAssault);
				unit.Command(&comm);
				comm.Comm(eComMove);
				comm.Vec(true,dloc);
			}
			else if ( !floc.empty() )   //move to nearest fort
			{
				comm.Comm(eComFormType);
				comm.Val(eFTMarch);
				unit.Command(&comm);
				comm.Comm(eComMove);
				comm.Vec(true,floc);
			} 
			else // i return in line
			{
				comm.Comm(eComFormType);
				comm.Val(eFTFight);
			}
		}
		else if ( (unit.EnemyClose() &&  unit.EnemyCloseYds() > (stance->SquareToEnemy * 2) ) || !unit.EnemyClose() )  //return to line
		{
			comm.Comm(eComFormType);
			comm.Val(eFTFight);
		}
	}
	else if ( state == eSTMarchCavalryDanger )  //the cavalry danger happen during the movement
	{
		if (unit.IsFormType(eFTAssault) && unit.TargClose() &&  unit.TargCloseYds() <= stance->SquareToEnemy )  //if I'm marching in assault formation I don't stop in the face of enemy
		{
			return false;
		}
		else if ( unit.OnDefTerr() && lead.StopOnDefTerr(unit) && lead.NewDefTerr() && !unit.TargClose() && !floc.empty())  //if no target close i try to reach a fortification
		{
			comm.Comm(eComMove);
			comm.Vec(true,floc);
		}
		else								//in general I halt to form the square
		{
			CXVec vec = unit.Loc();
			if (!unit.ValidWalk(unit.Loc()))   //if not valid pace I repath
			{
				unit.Repath();
				return true;
			}
			else if (unit.HaltTics() <= 0 && unit.ValidDest( unit.Loc() ) )  //halt
				comm.Comm(eComHalt);
			else if ( !unit.ValidDest( vec ) && unit.GetValidDest( unit.DeP(), &vec, true ) )  //new valid destination
			{
				comm.Comm(eComMove);
				comm.Vec(true,vec);
				unit.Command(&comm);
				comm.Comm(eComRun);
			}
		}
	}
//-----new square end
//--------------new forts----------------------------------------
	else if ( state == eSTStandFortMorale )   //morale check when inside a fort
	{
		if (unit.Morale()<stance->MoraleBesieged)  //check if the morale is sufficient  
		{
		   unit.SetClosest(NULL);
		   unit.SetMainTarg(NULL);
		   comm.Comm(eComRetreat);
		}
	}
	else if ( state == eSTStandFortTarget )   //action when inside a fort with target
	{
		if (unit.Ammo()<1)   //no ammo I leave the fort
		{
		   unit.SetClosest(NULL);
		   unit.SetMainTarg(NULL);
		   comm.Comm(eComGetAway);
		}
	}
	else if ( state == eSTStandFortNoTarget )   //action when inside a fort without target
	{
		if (unit.EnemyClose() && unit.EnemyCloseYds()<=(BUFF_CHECK(unit.MaxRangeYds())))
		{
			enemytoonear = true;
		}

		btarg = lead.HasTargets();
		if (unit.Ammo()==0 && lead.ShouldGetAmmo(unit))  //replenish ammunition
		{
			comm.Comm(eComResupply);
		}
		//the stance ask me to leave the fort but only if no enemy is too near
		else if (!btarg && !enemytoonear && stance->LeaveFort==1 && unit.Morale() >= stance->maintainline
			&& lead.GetLineDest( unit, pt ))  //leader has not target I check if he wants me out
		{
			comm.Comm(eComMove);
			comm.Vec(true,pt);
		}
	}
	else if ( state == eSTStandEnterFort )   //action for enter in a fort
	{
		if (unit.EnemyClose() && unit.EnemyCloseYds()<unit.MaxRangeYds())  //I cannot enter if enemy too near
			return false;
		
		if ( !floc.empty() && stance->LeaveFort!=1)  //I check the leader attitude to leave me enter
		{
			comm.Comm(eComMove);
			comm.Vec(true,floc);
		}
	}
//----new forts ----------------------------------------
//------new skirmishers---------------------------------
	else if ( state == eSTStandSkirmishTarget )   //skirmishers with target
	{
		
		if (!unit.TargFind(unit.Closest(),dist))
			dist = INT_MAX;


		if (unit.Ammo()==0 && lead.ShouldGetAmmo(unit))  //no ammunition if split I unsplit otherwise I go to search ammo
		{
			if (unit.Split()==-1)
			{
				comm.Comm(eComResupply);
			}
			else
			{
				comm.Comm(eComUnSplit);
			}
		}
		else if ( unit.Split()!=-1 && ( stance->SkirmAttitude != 1 || unit.Morale() < stance->SkirmSplit || lead.IsFormType(eFTAltSkirmish) ))  //skirmishers renter in the battalion
		{
				comm.Comm(eComUnSplit);
		}
		else if (unit.Closest() && dist<= unit.MaxRangeYds())  //target near I start to shoot
		{
			diff = unit.Pos().GetAngleDiff(unit.Closest().Loc(),true);  //check angle and wheel
			if (diff>30 && diff<330)
				unit.WheelToTarget(unit.Closest());
			else 
			{
				comm.Comm(eComFire);
			}
			return true;
		}
		else if (!unit.Closest())  //no frontal target I search best defensive position
		{
			if (unit.TargClose())  //wheel to the target
			{
				unit.WheelToTarget(unit.TargClose());
				return true;
			}
			else if ( !floc.empty() )   //enter in the fort
			{
				comm.Comm(eComMove);
				comm.Vec(true,floc);
			}
			else if (!unit.IsHighGrnd() && !unit.OnDefTerr())  //best ground in alternative
			{
				if (unit.FindClosestFeature(SKD_DBON,stance->fortrange,dloc))
				{
					comm.Comm(eComMove);
					comm.Vec(true,dloc);
				}
			}

		}
	}
	else if ( state == eSTStandSkirmishNoTarget )   //situation for skirmishers without targets
	{
		//---------------new skirmishers------------------------------------------------
		if ( !floc.empty() )    //enter in the fort
		{
			comm.Comm(eComMove);
			comm.Vec(true,floc);
		} 
		else if (unit.Split()!=-1 && !unit.CanRun())  //too tired to run
		{
			comm.Comm(eComUnSplit);  
		}
		else if (unit.Split()!=-1 && ( !unit.EnemyClose() || unit.EnemyCloseYds()>=(unit.MaxRangeYds() * 6)))  //enemy too distant, I reenter in the battalion
		{
			comm.Comm(eComUnSplit);
		}
		else if ( unit.Split()!=-1 && ( stance->SkirmAttitude != 1 || unit.Morale() < stance->SkirmSplit ))  //skirmishers renter in the battalion
		{
			comm.Comm(eComUnSplit);
		}
		else if (unit.Ammo()==0 && lead.ShouldGetAmmo(unit))  //no ammunition if split I unsplit otherwise I go to search ammo
		{
			if (unit.Split()==-1)
			{
				comm.Comm(eComResupply);
			}
			else
			{
				comm.Comm(eComUnSplit);
			}
		}
		else if (!unit.Blocked() && !unit.TargClose() && unit.EnemyClose() && unit.EnemyCloseYds()<(unit.MaxRangeYds() * 3) && lead.GetForwardDest(unit,pt,unit.EnemyClose()) )  //skirmishers attack own enemy close frontally
		{
			comm.Comm(eComMove);
			comm.Vec(true,pt);
		}
		else if (!unit.TargClose() && unit.EnemyClose() && unit.EnemyCloseYds()<(unit.MaxRangeYds() * 2) && lead.GetFlankDest(unit,pt,unit.EnemyClose()) )  //lateral attack
		{
			comm.Comm(eComMove);
			comm.Vec(true,pt);
		}
		else if (!unit.Blocked() && !unit.TargClose() && lead.EnemyClose() && lead.EnemyCloseYds()<(unit.MaxRangeYds() * 3) && lead.GetForwardDest(unit,pt,lead.EnemyClose()) )  //skirmishers has not own proper target, they attack the target of leader
		{
			comm.Comm(eComMove);
			comm.Vec(true,pt);
		}
		else if ( !unit.TargClose() && lead.EnemyClose() && lead.EnemyCloseYds()<(unit.MaxRangeYds() * 2) && lead.GetFlankDest(unit,pt,lead.EnemyClose()) )  //lateral attack
		{
			comm.Comm(eComMove);
			comm.Vec(true,pt);
		}
		//---------------new skirmishers----------------------------------------------
	}	
//----- new infantry-----------------------------------------------------
	else if ( state == eSTMarchTargetOnlyMelee )   //offensive movement
	{

		if (unit.Morale()>=stance->moveforward )  //check before I have the necessary morale 
		{
			if (!unit.EnemyFind(unit.Closest(),dist))
				dist = INT_MAX;

			//check of friend ahead me
			pathblocked = FriendAheadMe(unit);

			if (unit.IsFormType(eFTAssault) && lead.CanICharge(unit, stance->chargemelee) && !unit.Closest().InFort()
				&& dist<stance->InfantryCharge && unit.MorChange() >= 0  && unit.Closest().UnitType()!=eUnitCav
				&& ratio>=2)  //attack column sufficiently near I charge 
			{
				comm.Comm(eComCharge);
			}
			else if (IsLine(unit) &&  dist<= (unit.GetUnitTypeVal(UVAL_BaseVolleyYds)+unit.GetMod(eGameVolleyYds))
				&& !unit.Closest().InFort())  //i stop the line under the volley distance
			{
				if (unit.HaltTics() <= 0 && unit.ValidDest( unit.Loc() ) )
					comm.Comm(eComHalt);
			}
			else if (unit.IsFormType(eFTAssault) && dist > stance->InfantryCharge && !unit.Closest().InFort()
				&& lead.CanICharge(unit, stance->chargemelee)  
				&&  unit.MorChange() >= 0 && ratio>=2 && unit.ValidDest(unit.Closest().Loc())
				 && unit.Closest().UnitType()!=eUnitCav && !pathblocked)  //in assault column I move against the enemy till the infantry range
			{

				diff = unit.Pos().GetAngleDiff(unit.Closest().Loc(),true);  //check angle for avoid lateral movement
				if ( diff <= 10 || diff >=350)  //already on teh good path
					return true;
				else if ( diff <= 30 || diff >= 330) 
				{
					comm.Comm(eComMove);
					comm.Vec(true,unit.Closest().Loc());
				}
			}
			else if (IsLine(unit) &&  unit.TargCloseYds()> (unit.GetUnitTypeVal(UVAL_BaseVolleyYds)+unit.GetMod(eGameVolleyYds))
				&& !unit.Closest().InFort()
				&& unit.Closest().UnitType()!=eUnitCav && unit.ValidDest(unit.Closest().Loc())
				&& !pathblocked )  //in line I move to align with the target
			{
				diff = unit.Pos().GetAngleDiff(unit.Closest().Loc(),true);  //check angle for avoid lateral movement
				if ( diff <= 10 || diff >=350)  //already on the good path
					return true;
				else if ( diff <= 30 || diff >= 330) 
				{
					comm.Comm(eComMove);
					comm.Vec(true,unit.Closest().Loc());
				}
			}
			else if (IsLine(unit) &&  unit.TargCloseYds()>= unit.LongRangeYds()
				&& unit.Closest().InFort()
				&& !pathblocked && unit.ValidDest(unit.GetLocAhead(20)))  //in line I move to align with the target
			{
				comm.Comm(eComMove);
				comm.Vec(true,unit.GetLocAhead(20));
			}
			else if (IsLine(unit) && unit.TargCloseYds() > unit.MaxRangeYds() )  //I continue to march
				return true;
		}

	}
	else if ( state == eSTMarchNoTarget )   //situation of march without a target
	{
			int brigform = stance->brigform;
		    comm.Comm(eComFormType);    //formation for the regiment coherent with that of brigade
			comm.Val(brigform);
	}
	else if ( state == eSTStandTarget )   //situation of standing with target
	{
		if ( unit.Morale()>=stance->moveforward && !IsSquareType(unit,true,false) )  //check before I have the necessary morale (square are immobile) and the target is not in a fort
		{		
			dist = unit.TargCloseYds();

			//check of friend ahead me
			pathblocked = FriendAheadMe(unit);

			//i'm in column i check if I can attack
			if (unit.IsFormType(eFTAssault) && lead.CanICharge(unit, stance->chargemelee) 
				&&  unit.MorChange() >= 0 && ratio>=2 && unit.Closest().UnitType()!=eUnitCav
				&& unit.ValidDest(unit.Closest().Loc()) && !unit.Closest().InFort() && !pathblocked)
			{
				
				if (dist > stance->InfantryCharge)   //move
				{
					diff = unit.Pos().GetAngleDiff(unit.Closest().Loc(),true);  //check angle for avoid lateral movement
					if ( diff <= 30 || diff >= 330) 
					{				
						comm.Comm(eComMove);
						comm.Vec(true,unit.Closest().Loc());
					}
				}
				else
					comm.Comm(eComCharge);  //charge
			}
			else if (IsLine(unit) && !unit.Closest().InFort()
				&& ( unit.VolleyStatus()==1 || (unit.State()==eUnitStand && !unit.UnderFire1()) 
				|| IsSkirmisher(unit.Closest()))     // linear advance to shot a volley , if not shooting or for free herself from the skirmishers
				&&  !unit.Closest().Marching()
				&& dist > (unit.GetUnitTypeVal(UVAL_BaseVolleyYds)+unit.GetMod(eGameVolleyYds)) 
				&& unit.Closest().UnitType()!=eUnitCav
				&& unit.ValidDest(unit.Closest().Loc())
				&& !pathblocked)
			{
				diff = unit.Pos().GetAngleDiff(unit.Closest().Loc(),true);  //check angle for avoid lateral movement
				if ( diff <= 30 || diff >= 330) 
				{
					comm.Comm(eComMove);
					comm.Vec(true,unit.Closest().Loc());
				}
			}
			else if (IsLine(unit) && unit.Closest().InFort() //advance to the fort attack
				&& dist >= unit.LongRangeYds() 
				&& !pathblocked && unit.ValidDest(unit.GetLocAhead(20)))
			{
				comm.Comm(eComMove);
				comm.Vec(true,unit.GetLocAhead(20));
			}
		}
		else if (!unit.Closest().InFort())  //short distnace fire from standing position
		{
			int targdist = 0;
			if (!unit.TargFind(unit.Closest(),targdist))
				targdist = INT_MAX;
			
		//enemy is not standing\shooting (marching against me), there is not charge and is over the minimal distance
			//prepare for short distance fire
			if ( (IsLine(unit) || IsSquareType(unit,true,false) ) && unit.Closest().Marching()
				&& targdist>(unit.GetUnitTypeVal(UVAL_BaseVolleyYds)+unit.GetMod(eGameVolleyYds)) && !unit.GetCharger()
				&& !unit.UnderFire1())
			{
				return true;
			}
		}
	}
	else if ( state == eSTStandNoWheel )   //stand and not wheel for hold to the last order
	{
		if ( lead.GetOrders()->regwheel )
			return true;

		return false;

	}
//--------new infantry-----------------------------
	return comm.Comm() != eComMax;
}

//*************************************************************************************************************
//*************************************************************************************************************
// Function: GetCommand
//*************************************************************************************************************
//*************************************************************************************************************

bool GetCavCommand( CXOff lead, int state, SXComm &comm, CXUnit unit )
{
	int  dist;
	bool bMoved = false;

// ---- new cavalry----
	bool enemytoonear = false;
// ---- new cavalry-----

	CXVec pt;
	CXVec dir;

	CXUnit targ;

	const TStance *stance;

	comm.Comm(eComMax);

	if ( !lead.HasStance() || lead.PlyrCtrl() )
	{
		
		
		
		
		return false;
	}

	stance = lead.GetStance();

	if ( state == eSTStandNoTargetFire )
	{
		if ( unit.TCOn() )
			return false;

		if ( unit.Morale() <= stance->retreat + 1 )
		{
			comm.Comm(eComGetAway);

			bMoved = true;
		}
//------new cavalry------------------------------------
		else if ( unit.UnderFire1() || unit.IsFlanked() || unit.UnderRecentFire())  //if in unpleasant situation move back out of range
		{
			comm.Comm(eComMoveDir);
			CXVec vec = unit.GetLocBehind(150);
			if ( !unit.ValidDest(vec))
			{
				unit.GetValidDest( unit.DeP(), &vec, true );
			}
			comm.Vec(true,vec);
			comm.Dir(true,unit.Dir());
			unit.Command(&comm);
			comm.Comm( eComRun );
			bMoved = true;
		}
//------end new cavalry--------------------------------
	}
	else if ( state == eSTStandNoTarget )
	{
		if ( unit.Morale() <= MORAL_CANTWHEEL || unit.TCOn() )
			return false;

		if ( lead.HasTargets() )
		{
			//new cavalry : i want avoid manouvers differents from the normal advance with the enemy too near
			if (unit.EnemyClose() && unit.EnemyCloseYds()<=(BUFF_CHECK(stance->CavRunDist)))
			{
				enemytoonear = true;
			}

			// Flanking
			// targ is empty here, so the officer will find the best target
			if ( unit.Morale() >= stance->flanking && lead.GetFlankDest( unit, pt, targ ) && !enemytoonear)
			{
				comm.Vec (true,pt);
				comm.Comm(eComMove);

				bMoved = true;
			}

			// Maintain line			
			if ( !bMoved && unit.Morale() >= stance->maintainline && lead.GetMaintainDest( unit, pt, dir ) && !enemytoonear )
			{
				comm.Vec (true, pt);
				comm.Dir (true, dir);
				comm.Comm(eComMoveDir);
			}
		}
	}
	else if ( state == eSTStandShooting )
	{
		if ( lead.ShouldGetAmmo( unit ) )
			comm.Comm( eComGetAway );
		else if ( unit.Morale() <= stance->retreat && unit.MorChange() < 0 )
			comm.Comm( eComRetreat );
		else if ( unit.OnDefTerr() && lead.StopOnDefTerr(unit) && lead.NewDefTerr() )
			return false;
		else if ( unit.Morale() <= stance->fback && unit.MorChange() < 0 )
			comm.Comm( eComFallback );
		else if ( !unit.PlyrCtrl() && unit.Closest().Valid() )
		{
			targ = unit.Closest();
			
			unit.TargFind( targ, dist );

			lead.GetOrders( dist, targ, unit );
		}
	}
	else if ( state == eSTStandTargetNoAmmo )
	{
		if ( lead.CanICharge( unit, stance->chargenoammo / 2 ) )
			comm.Comm( eComCharge );
		else
			comm.Comm( eComGetAway );
	}
	else if ( state == eSTStandTargetOnlyMelee )
	{
		if ( lead.CanICharge( unit, stance->chargemelee / 2 ) )
			comm.Comm( eComCharge );
	}
//-------new cavalry------------------------------
	else if ( state == eSTAttackTarget )    //can I charge to the target
	{
		if (!unit.EnemyFind(unit.Closest(),dist))
			dist = INT_MAX;

		CXUnit charger = unit.GetCharger();
		bool avoidcharge = false;

		//If I give charge with a target close different from the frontal one I risk to charge a square
		if (unit.TargClose() && unit.TargClose().GetID()!=unit.Closest().GetID() && unit.TargCloseYds() < stance->CavChargDist && IsSquareType(unit.TargClose(),true,false))
			avoidcharge = true;

//to charge particular type of targets, unit has to be already in running state		
		if (  lead.CanICharge( unit, stance->CavCharge ) && !avoidcharge ) 
		{
			if (unit.TargClose() && unit.DoesShareHex(unit.TargClose()))   //if happen to have contact
			{
				unit.SetClosest(unit.TargClose());
				comm.Comm( eComState );
				comm.Val( eUnitMelee );
			}
			else if (IsSkirmisher(unit))  //skirmishers can charge only skirmishers or artillery no distance check is a uncoordinate charge
			{
				if (IsSkirmisher(unit.Closest()) || unit.Closest().IsArt())
				{
					comm.Comm( eComCharge );
				}
			}
			else if (unit.TargClose() && unit.TargClose().GetID()!=unit.Closest().GetID()
				&& ( IsSquareType(unit.Closest(),true,false) || dist >= stance->CavChargDist )
				&& unit.TargCloseYds() < (DISTCAV1AUTOMELEE/2) && !IsSquareType(unit.TargClose(),true,false) && !unit.TargClose().InFort()) //to avoid to not charge enemy too near to not be charged if the main target is far from being charged
			{
				unit.SetClosest(unit.TargClose());
				comm.Comm( eComCharge );
			}
			else if ( IsSquareType(unit.Closest(),true,false) && !unit.Closest().Retreating() && dist < stance->CavChargDist ) //target is in square 
			{
				if ( (unit.NumMen() *  unit.Morale())  >  (unit.Closest().NumMen() * unit.Closest().Morale() * stance->SquareChargeFactor ) && unit.State()==eUnitRun )  //I charge the target in square only if I have good probably to broke it
				{
					comm.Comm( eComCharge );
				}
				else
					return false;  //I stop
			}
			else if ( unit.Closest().Valid() && dist < stance->CavChargDist)  //charge target
			{
				comm.Comm( eComCharge );
			}
			else if (charger!=NULL && charger.GetID() == unit.Closest().GetID()) //countercharge from marching
			{
				comm.Comm( eComCharge );
			}
		}
		else   //I cannot charge
		{
			if (unit.TargClose() && unit.DoesShareHex(unit.TargClose()))   //if happen to have contact
			{
				unit.SetClosest(unit.TargClose());
				comm.Comm( eComState );
				comm.Val( eUnitMelee );
			}
			else if (charger!=NULL) //countercharge from marching IF i CANNOT COUNTER CHARGE i GET AWAY
			{
				unit.SetClosest(NULL);
				unit.SetMainTarg(NULL);
				comm.Comm( eComGetAway );					
			}
			else if (unit.TargClose() && unit.TargCloseYds() < (DISTCAV1AUTOMELEE/2) && !IsSkirmisher(unit) && !IsSquareType(unit.TargClose(),true,false) && !unit.TargClose().InFort()) //to avoid to not charge enemy too near to not be charged
			{
				unit.SetClosest(unit.TargClose());
				comm.Comm( eComCharge );
			}
			else if (dist < stance->CavChargDist )   //move back for avoid to remain in the face of enemy
			{						
				unit.SetClosest(NULL);
				unit.SetMainTarg(NULL);
				comm.Comm(eComMoveDir);
				CXVec vec = unit.GetLocBehind(150);
				if ( !unit.ValidDest(vec))
				{
					unit.GetValidDest( unit.DeP(), &vec, true );
				}
				comm.Vec(true,vec);
				comm.Dir(true,unit.Dir());
				unit.Command(&comm);
				comm.Comm( eComRun );
			}
			else   //I stop
			{
				unit.SetClosest(NULL);
				unit.SetMainTarg(NULL);
				comm.Comm( eComHalt );
			}
		}

	}
	else if ( state == eSTMarchTargetOnlyMelee )    //can I start to trot\gallop
	{
		if (!unit.EnemyFind(unit.Closest(),dist))
			dist = INT_MAX;

		int ratio = FearCheck(unit.Closest(),unit);
		int diff ;

		bool stopenemytoonear = false;
		bool pathagain = false;
		bool pathblocked = false;

		if (unit.TargClose() && unit.TargClose().GetID()!=unit.Closest().GetID())
		{
			diff = unit.Pos().GetAngleDiff(unit.TargClose().Loc(),true);
			if (diff<=45 && diff>=315)
				pathblocked = true;
		}

		//If I give charge with a target close different from the frontal one I risk to charge a square
		if (pathblocked && unit.TargCloseYds() <= stance->CavChargDist && unit.TargCloseYds() < dist)
			stopenemytoonear = true;

		//If I give charge with a target close different from the frontal one I risk to charge a square
		if (pathblocked && unit.TargCloseYds() > stance->CavChargDist && unit.TargCloseYds() < dist)
			pathagain = true;

		if (unit.TargClose() && unit.DoesShareHex(unit.TargClose()))   //if happen to have contact
		{
			unit.SetClosest(unit.TargClose());
			comm.Comm( eComState );
			comm.Val( eUnitMelee );
		}
		else if (unit.IsFormType(eFTMarch) && dist > stance->CavRunDist)  //if still in march formation I change if I can
		{
			comm.Comm( eComFormType );
			comm.Val( eFTFight );
		}
		else if (!unit.ValidDest(unit.Loc()) || unit.Blocked() || stopenemytoonear)
		{
			comm.Comm( eComHalt );
		}
		else if (IsSquareType(unit.Closest(),true,false) && !unit.Closest().Retreating() && dist < stance->CavChargDist ) //additional check for stop
		{
			comm.Comm( eComHalt );
		}
		else if (pathagain)  //flanking effect
		{
			unit.Repath();
			return true;
		}
		else if ( lead.CanICharge( unit, stance->CavCharge ) && (ratio>= 1 || unit.Closest().Retreating()) && !unit.Blocked() && !IsSquareType(unit.Closest(),true,false)
			&& !unit.Closest().InFort() && unit.ValidDest(unit.Closest().Loc()) && lead.CanICharge( unit, stance->CavCharge ) )
		{
			comm.Comm(eComMove);
			comm.Vec(true,unit.Closest().Loc());
			if (dist < stance->CavRunDist && unit.State()!=eUnitRun)
			{
				unit.Command(&comm);
				comm.Comm( eComRun );
			}
		}

	}
	else if ( state == eSTStandTarget )    //with a target to charge i'm standing
	{
	
		if (!unit.EnemyFind(unit.Closest(),dist))
			dist = INT_MAX;
		int diff;

		bool stopenemytoonear = false;
		bool pathblocked = false;

		if (unit.TargClose() && unit.TargClose().GetID()!=unit.Closest().GetID())
		{
			diff = unit.Pos().GetAngleDiff(unit.TargClose().Loc(),true);
			if (diff<=45 && diff>=315)
				pathblocked = true;
		}

		//If I give charge with a target close different from the frontal one I risk to charge a square
		if (pathblocked && unit.TargCloseYds() <= stance->CavChargDist && unit.TargCloseYds() < dist)
			stopenemytoonear = true;

		int ratio = FearCheck(unit.Closest(),unit);

		 //for particualr type of target need to be sure to be in fighintg formation or over a specific distance
		if (unit.TargClose() && unit.DoesShareHex(unit.TargClose()))   //if happen to have contact
		{
			unit.SetClosest(unit.TargClose());
			comm.Comm( eComState );
			comm.Val( eUnitMelee );
		} 
		else if ( !unit.IsFormType(eFTFight) && !IsSkirmisher(unit.Closest()) && !unit.Closest().IsArt() && dist > stance->CavChargDist)
		{
			comm.Comm( eComFormType );
			comm.Val( eFTFight );
		}
		else if (stopenemytoonear)
		{
			unit.WheelToTarget(unit.TargClose());
			return true;
		}
		else if ( (ratio>= 1 || unit.Closest().Retreating()) && lead.CanICharge( unit, stance->CavCharge ) && ( unit.NumMen() * unit.Morale() * stance->CavPowerFactor ) > (unit.Closest().NumMen() * unit.Closest().Morale())
			&& !IsSquareType(unit.Closest(),true,false) && !unit.Closest().InFort())  //check attack start
		{	
			if (!unit.Blocked() && unit.ValidDest(unit.Closest().Loc()))
			{
				comm.Comm( eComMove );
				comm.Vec(true,unit.Closest().Loc());
			}

		}
	}
	else if ( state ==eSTMeleeTargetSquare )    //Cavalry melee to manage if I'm enter in melee with a square
	{
		int ratio = FearCheck(unit.Closest(),unit);
		
		if ( IsSquareType(unit.Closest(),true,false) && !unit.Closest().Retreating() && (unit.NumMen() *  unit.Morale())  <=  (unit.Closest().NumMen() * unit.Closest().Morale() * stance->SquareChargeFactor )) 
		{
			unit.SetClosest(NULL);
			unit.SetMainTarg(NULL);				
			comm.Comm( eComGetAway );	
		}
		else if (ratio<1 && !unit.Closest().Retreating())
		{
			unit.SetClosest(NULL);
			unit.SetMainTarg(NULL);				
			comm.Comm( eComGetAway );	
		}

	}
	else if ( state ==eSTChargeTargetSquare )    //Cavalry melee to manage if I'm enter in melee with a square
	{
		int ratio = FearCheck(unit.Closest(),unit);

		if ( unit.Closest().Retreating() && !lead.CanICharge( unit, stance->CavCharge ))  //retreating target I can continue?
		{
				unit.SetClosest(NULL);
				unit.SetMainTarg(NULL);
				comm.Comm( eComHalt );
		}
		else if (unit.Closest().Retreating()
			&& ((unit.TargClose() && !unit.TargClose().Retreating() && unit.TargCloseYds()<unit.GetUnitTypeVal(UVAL_MarchTargYds))
			|| !unit.ValidDest(unit.Loc()) || unit.UnderRecentFire()))
		{
				unit.SetClosest(NULL);
				unit.SetMainTarg(NULL);
				comm.Comm( eComHalt );
		}
		else if  ( IsSquareType(unit.Closest(),true,false) &&  (unit.NumMen() *  unit.Morale())  <=  (unit.Closest().NumMen() * unit.Closest().Morale() * stance->SquareChargeFactor ) )  
		{
			unit.SetClosest(NULL);
			unit.SetMainTarg(NULL);
			comm.Comm( eComHalt );
		}
		else if (ratio<1 && !unit.Closest().Retreating())
		{
			unit.SetClosest(NULL);
			unit.SetMainTarg(NULL);				
			comm.Comm( eComHalt );	
		}
	}
	else if ( state == eSTMarchNoTarget )   
	{
			int brigform = stance->brigform;
			comm.Comm(eComFormType);    //formation for the regiment coherent with that of brigade
			comm.Val(brigform);
	}
//-------end new cavalry--------------------------
	return comm.Comm() != eComMax;
}

//*************************************************************************************************************
//*************************************************************************************************************
// Function: GetCommand
//*************************************************************************************************************
//*************************************************************************************************************

bool GetArtCommand( CXOff lead, int state, SXComm &comm, CXUnit unit )
{
	bool bMoved = false;

	CXVec pt;
	CXVec dir;
	int fallback;
	int redeploydst = 10;

	comm.Comm(eComMax);

	const TStance *stance;

	if ( !lead.HasStance() || lead.PlyrCtrl() )
		return false;

	stance = lead.GetStance();

	if ( state == eSTStandNoTarget )
	{
		if ( unit.Morale() <= MORAL_CANTWHEEL || unit.TCOn() )
			return false;

		if ( lead.HasTargets() )
		{
			// Maintain line			
			if ( !bMoved && unit.Morale() >= stance->maintainline && lead.GetMaintainDest(unit,pt,dir) )
			{				
			   if (CanGunRedeploy(lead,pt)) //if destination is not good I avoid the change
			   {				
					comm.Comm( eComMoveDir );
					comm.Vec(true,pt);
					comm.Dir(true,dir);
			   }
			}

			if ( !bMoved && comm.Comm() == eComMax )
			{
				if (unit.Blocked() && unit.EnemyClose().Valid())  
				{
					if (unit.LimState()==eLimLimbered && unit.WheelToTarget(unit.EnemyClose())) //limbered troops can face the wrong direction, I want to be sure they face the enemy before move back
						return true;
					if (unit.LimState()==eLimLimbered)  //limbered troops can need more space after because unlimbered they take more space
						redeploydst = 20;
					comm.Comm( eComMoveDir );
					comm.Vec(true,unit.GetLocBehind(redeploydst));
					comm.Dir(true,unit.Dir());
				}
			}

		}
	}
	else if ( state == eSTStandShooting )
	{
		if ( lead.ShouldGetAmmo( unit ) )
			comm.Comm( eComRetreat );
		else if ( unit.Morale() <= stance->retreat && unit.MorChange() < 0 )
			comm.Comm( eComRetreat );
		else if ( unit.OnDefTerr() && lead.StopOnDefTerr(unit) && lead.NewDefTerr() )
			return false;
		else if ( unit.Morale() <= stance->fback && unit.MorChange() < 0 )
			comm.Comm( eComFallback );
	}
	else if ( state == eSTStandTargetNoAmmo )
		comm.Comm( eComRetreat );
	else if ( state == eSTStandTargetOnlyMelee )
	{
	}
	else if ( state == eSTStandArtilleryFallBack )
	{
		fallback = stance->ArtyFallback;
		comm.Val(fallback);
		return true;
	}
	return comm.Comm() != eComMax;
}

//*************************************************************************************************************
//*************************************************************************************************************
//*************************************************************************************************************
//*************************************************************************************************************


